@@ -25,25 +25,17 @@ typedef short PyInt16;
 #endif
 
 static const int maxvals[] = {0, 0x7F, 0x7FFF, 0x7FFFFF, 0x7FFFFFFF};
-/* -1 trick is needed on Windows to support -0x80000000 without a warning */
-static const int minvals[] = {0, -0x80, -0x8000, -0x800000, -0x7FFFFFFF-1};
+static const int minvals[] = {0, -0x80, -0x8000, -0x800000, -0x80000000};
 static const unsigned int masks[] = {0, 0xFF, 0xFFFF, 0xFFFFFF, 0xFFFFFFFF};
 
 static int
 fbound(double val, double minval, double maxval)
 {
-    if (val > maxval) {
+    if (val > maxval)
         val = maxval;
-    }
-    else if (val < minval + 1.0) {
+    else if (val < minval + 1)
         val = minval;
-    }
-
-    /* Round towards minus infinity (-inf) */
-    val = floor(val);
-
-    /* Cast double to integer: round towards zero */
-    return (int)val;
+    return val;
 }
 
 
@@ -129,7 +121,7 @@ static PyInt16 _st_ulaw2linear16[256] = {
 
 /*
  * linear2ulaw() accepts a 14-bit signed integer and encodes it as u-law data
- * stored in an unsigned char.  This function should only be called with
+ * stored in a unsigned char.  This function should only be called with
  * the data shifted such that it only contains information in the lower
  * 14-bits.
  *
@@ -394,9 +386,7 @@ audioop_minmax(PyObject *self, PyObject *args)
     signed char *cp;
     int len, size, val = 0;
     int i;
-    /* -1 trick below is needed on Windows to support -0x80000000 without
-    a warning */
-    int min = 0x7fffffff, max = -0x7FFFFFFF-1;
+    int min = 0x7fffffff, max = -0x80000000;
 
     if (!PyArg_ParseTuple(args, "s#i:minmax", &cp, &len, &size))
         return NULL;
@@ -803,8 +793,8 @@ audioop_mul(PyObject *self, PyObject *args)
         if ( size == 1 )      val = (int)*CHARP(cp, i);
         else if ( size == 2 ) val = (int)*SHORTP(cp, i);
         else if ( size == 4 ) val = (int)*LONGP(cp, i);
-        fval = (double)val * factor;
-        val = fbound(fval, minval, maxval);
+        fval = (double)val*factor;
+        val = (int)floor(fbound(fval, minval, maxval));
         if ( size == 1 )      *CHARP(ncp, i) = (signed char)val;
         else if ( size == 2 ) *SHORTP(ncp, i) = (short)val;
         else if ( size == 4 ) *LONGP(ncp, i) = (Py_Int32)val;
@@ -847,8 +837,8 @@ audioop_tomono(PyObject *self, PyObject *args)
         if ( size == 1 )      val2 = (int)*CHARP(cp, i+1);
         else if ( size == 2 ) val2 = (int)*SHORTP(cp, i+2);
         else if ( size == 4 ) val2 = (int)*LONGP(cp, i+4);
-        fval = (double)val1 * fac1 + (double)val2 * fac2;
-        val1 = fbound(fval, minval, maxval);
+        fval = (double)val1*fac1 + (double)val2*fac2;
+        val1 = (int)floor(fbound(fval, minval, maxval));
         if ( size == 1 )      *CHARP(ncp, i/2) = (signed char)val1;
         else if ( size == 2 ) *SHORTP(ncp, i/2) = (short)val1;
         else if ( size == 4 ) *LONGP(ncp, i/2)= (Py_Int32)val1;
@@ -891,11 +881,11 @@ audioop_tostereo(PyObject *self, PyObject *args)
         else if ( size == 2 ) val = (int)*SHORTP(cp, i);
         else if ( size == 4 ) val = (int)*LONGP(cp, i);
 
-        fval = (double)val * fac1;
-        val1 = fbound(fval, minval, maxval);
+        fval = (double)val*fac1;
+        val1 = (int)floor(fbound(fval, minval, maxval));
 
-        fval = (double)val * fac2;
-        val2 = fbound(fval, minval, maxval);
+        fval = (double)val*fac2;
+        val2 = (int)floor(fbound(fval, minval, maxval));
 
         if ( size == 1 )      *CHARP(ncp, i*2) = (signed char)val1;
         else if ( size == 2 ) *SHORTP(ncp, i*2) = (short)val1;
@@ -954,7 +944,7 @@ audioop_add(PyObject *self, PyObject *args)
         else {
             double fval = (double)val1 + (double)val2;
             /* truncate in case of overflow */
-            newval = fbound(fval, minval, maxval);
+            newval = (int)floor(fbound(fval, minval, maxval));
         }
 
         if ( size == 1 )      *CHARP(ncp, i) = (signed char)newval;
@@ -1096,7 +1086,7 @@ audioop_ratecv(PyObject *self, PyObject *args)
     char *cp, *ncp;
     int len, size, nchannels, inrate, outrate, weightA, weightB;
     int chan, d, *prev_i, *cur_i, cur_o;
-    PyObject *state, *samps, *str, *rv = NULL, *channel;
+    PyObject *state, *samps, *str, *rv = NULL;
     int bytes_per_frame;
 
     weightA = 1;
@@ -1162,10 +1152,6 @@ audioop_ratecv(PyObject *self, PyObject *args)
             prev_i[chan] = cur_i[chan] = 0;
     }
     else {
-        if (!PyTuple_Check(state)) {
-            PyErr_SetString(PyExc_TypeError, "state must be a tuple or None");
-            goto exit;
-        }
         if (!PyArg_ParseTuple(state,
                         "iO!;audioop.ratecv: illegal state argument",
                         &d, &PyTuple_Type, &samps))
@@ -1176,13 +1162,7 @@ audioop_ratecv(PyObject *self, PyObject *args)
             goto exit;
         }
         for (chan = 0; chan < nchannels; chan++) {
-            channel = PyTuple_GetItem(samps, chan);
-            if (!PyTuple_Check(channel)) {
-                PyErr_SetString(PyExc_TypeError,
-                                "ratecv(): illegal state argument");
-                goto exit;
-            }
-            if (!PyArg_ParseTuple(channel,
+            if (!PyArg_ParseTuple(PyTuple_GetItem(samps, chan),
                                   "ii:ratecv", &prev_i[chan],
                                                &cur_i[chan]))
                 goto exit;
